/*
 * Created on June 8, 2005
 * @author Rafael Santos (rafael.santos@lac.inpe.br)
 * 
 * Part of the Java Advanced Imaging Stuff site
 * (http://www.lac.inpe.br/~rafael.santos/Java/JAI)
 * 
 * STATUS: Complete.
 * 
 * Redistribution and usage conditions must be done under the
 * Creative Commons license:
 * English: http://creativecommons.org/licenses/by-nc-sa/2.0/br/deed.en
 * Portuguese: http://creativecommons.org/licenses/by-nc-sa/2.0/br/deed.pt
 * More information on design and applications are on the projects' page
 * (http://www.lac.inpe.br/~rafael.santos/Java/JAI).
 */

import java.awt.BorderLayout;
import java.awt.image.renderable.ParameterBlock;
import javax.media.jai.Histogram;
import javax.media.jai.JAI;
import javax.media.jai.PlanarImage;
import javax.swing.JFrame;

/**
 * This class uses the DisplayHistogram component in an application.
 */
public class DisplayHistogramApp extends JFrame 
  {
  private DisplayHistogram dh; // an instance of the DisplayHistogram component

 /**
  * The constructor for the class, which will create the user interface and
  * register the mouse motion events.
  */
  public DisplayHistogramApp(String name,Histogram histo)
    {
    setTitle("Histogram of "+name);
    getContentPane().setLayout(new BorderLayout());
    // Add to this ContentPane an instance of DisplayHistogram, which will
    // contain the histogram plot.
    dh = new DisplayHistogram(histo,"Histogram of "+name);

   dh.setBinWidth(8); dh.setHeight(160); dh.setIndexMultiplier(4); 
   dh.setSkipIndexes(2);

    getContentPane().add(dh,BorderLayout.CENTER);
    // Set the closing operation so the application is finished.
    setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    pack(); // adjust the frame size using preferred dimensions.
    setVisible(true); // show the frame.
    }

 /**
  * The application entry point.
  * @param args the command line arguments.
  */
  public static void main(String[] args)
    {
    PlanarImage image = JAI.create("fileload", args[0]);
    ParameterBlock pb = new ParameterBlock();
    pb.addSource(image);
    pb.add(null); // The ROI.
/*
    pb.add(1); // Samplings.
    pb.add(1);
    pb.add(new int[]{256}); // Num. bins.
    pb.add(new double[]{0}); // Min value to be considered.
    pb.add(new double[]{256}); // Max value to be considered.
*/

    pb.add(1); pb.add(1); // Sampling
    pb.add(new int[]{64}); // Bins
    pb.add(new double[]{0});  pb.add(new double[]{256}); // Range for inclusion


    // Creates the histogram.
    PlanarImage temp = JAI.create("histogram", pb);
    javax.media.jai.Histogram h = (Histogram)temp.getProperty("histogram");

    System.out.println("getNumBands()="+h.getNumBands());
    double[] mean = h.getMean();
    double[] rms =  h.getStandardDeviation();


    for (int i=0; i<h.getNumBands(); i++){
     System.out.println(i+" getMean()="+mean[i]);
     System.out.println(i+" getStandardDeviation()="+rms[i]);
     System.out.println(i+" getHighValue()="+h.getHighValue(i)); 
    }

    int binWidth = 3;
     // Calculate the components dimensions.
    double width = h.getNumBins(0)*binWidth;
    // Get the histogram data.
    int  [] counts = h.getBins(0);
    // Get the max and min counts.
    int  maxCount = Integer.MIN_VALUE;
    for(int c=0;c<counts.length;c++) maxCount = Math.max(maxCount,counts[c]);

   
/* 
     for(int bin=0;bin<h.getNumBins(0);bin++)
      {
      int x = 2+bin*4;
      double barEnds = counts[bin];
      System.out.println(bin+ " "+x+"  "+barEnds);
      }
*/

     H1D hh = new H1D("test",h.getNumBins(0), 0,256);
     hh.setHights(counts);
     hh.normilize();
     hh.show();
 
    // javax.media.jai.Histogram h1=h.getGaussianSmoothed(5); 
    // new DisplayHistogramApp(args[0],h1);
       new DisplayHistogramApp(args[0],h);

    }

  } // end class
