/*
 * Created on Jun 9, 2005
 * @author Rafael Santos (rafael.santos@lac.inpe.br)
 * 
 * Part of the Java Advanced Imaging Stuff site
 * (http://www.lac.inpe.br/~rafael.santos/Java/JAI)
 * 
 * STATUS: Complete, but will probably add some features in the future.
 * 
 * Redistribution and usage conditions must be done under the
 * Creative Commons license:
 * English: http://creativecommons.org/licenses/by-nc-sa/2.0/br/deed.en
 * Portuguese: http://creativecommons.org/licenses/by-nc-sa/2.0/br/deed.pt
 * More information on design and applications are on the projects' page
 * (http://www.lac.inpe.br/~rafael.santos/Java/JAI).
 */

/**
 * This class represents the information about a cluster. Instances of it will
 * be returned by a clustering task.
 */
public class KMeansClusterDescriptor
  {
  // The cluster description fields: a centroid and the number of assigned points or pixels.
  private float[]   centroid;
  private float     centroidX; 
  private float     centroidY;
  private double    compactness;
  private double    compactness_col;

  private short[][] clusterindex;
  private int numberOfAssignedPoints;
  
 /**
  * Returns the centroid of this cluster as a array of doubles.
  * @return the cluster's centroid.
  */ 
  public float[] getCentroid()
    {
    return centroid;
    }
 

/**
  * Returns the centroid of this cluster as a array in X.
  * @return the cluster's centroid.
  */
  public float getCentroidX()
    {
    return centroidX;
    }


/**
  * Returns the centroid of this cluster as a array in Y.
  * @return the cluster's centroid.
  */
  public float getCentroidY()
    {
    return centroidY;
    }


/**
  * Returns the compactness.
  * @return the cluster's compactness.
  */
  public double getCompactness()
    {
    return compactness;
    }

/**
  * Returns the compactness in color.
  * @return the cluster's compactness.
  */
  public double getCompactnessCol()
    {
    return compactness_col;
    }

 
 /**
  * Set the centroid values of this cluster.
  * @param centroid the array with the centroid values.
  */
  public void setCentroid(float[] centroid)
    {
    this.centroid = centroid;
    }

/**
  * Set the centroid values of this cluster in X and Y space.
  * @param centroid the array with the centroid values.
  */
  public void setCentroidXY(float centroidX, float centroidY)
    {
    this.centroidX = centroidX; this.centroidY = centroidY; 
    }

/**
  * Set compactness. 
  */
  public void setCompactness(double compactness)
    {
    this.compactness = compactness;
    }

/**
  * Set compactness.
  */
  public void setCompactnessCol(double compactness_col)
    {
    this.compactness_col = compactness_col;
    }

 
/**
  * Set the cluster index 
  * @param clusterindex the array with cluster index.
  */
  public void setClusterIndex(short[][] clusterindex)
    {
    this.clusterindex = clusterindex;
    }

 
 /**
  * Get the number of points with are assigned to this cluster.
  * @return the assignment count.
  */
  public int getNumberOfAssignedPoints()
    {
    return numberOfAssignedPoints;
    }
 
/**
  * Get the number of points with are assigned to this cluster.
  * @return getClusterIndex 
  */
  public short[][] getClusterIndex()
    {
    return clusterindex;
    }

 
 /**
  * Set the number of points with are assigned to this cluster.
  * @param numberOfAssignedPoints the new assignment count.
  */
  public void setNumberOfAssignedPoints(int numberOfAssignedPoints)
    {
    this.numberOfAssignedPoints = numberOfAssignedPoints;
    }
  
 /**
  * Returns the information about this cluster as a String.
  */
  public String toString()
    {
    String r = "Center [";
    for(int d=0;d<centroid.length;d++) r += (int)centroid[d]+" ";
    r +=" ] count:"+numberOfAssignedPoints;
    r +=" compactXY="+compactness;
    r +=" compactCOL="+compactness_col;
    r +=" pos= "+centroidX+",";
    r +=" "+centroidY;
    return r;
    }
 
/**
  * print index 
  */
  public void printIndex()
    {
    }








 
  }
