/*
 * Created on May 22, 2005
 * @author Rafael Santos (rafael.santos@lac.inpe.br)
 * 
 * Part of the Java Advanced Imaging Stuff site
 * (http://www.lac.inpe.br/~rafael.santos/Java/JAI)
 * 
 * STATUS: Complete.
 * 
 * Redistribution and usage conditions must be done under the
 * Creative Commons license:
 * English: http://creativecommons.org/licenses/by-nc-sa/2.0/br/deed.en
 * Portuguese: http://creativecommons.org/licenses/by-nc-sa/2.0/br/deed.pt
 * More information on design and applications are on the projects' page
 * (http://www.lac.inpe.br/~rafael.santos/Java/JAI).
 */
package operators;

import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.image.renderable.ParameterBlock;
import javax.media.jai.JAI;
import javax.media.jai.PlanarImage;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JPanel;
import display.DisplayThreeSynchronizedImages;

/**
 * This class demonstrates the use of several arithmetic operators using two
 * images to generate a third one. The three images will be shown, side by
 * side.
 */
public class ArithmeticOps extends JFrame implements ActionListener
  {
  // Four buttons, one for each operation.
  private JButton add,subtract,multiply,divide;
  // Three instances of PlanarImage: two input and one output images.
  private PlanarImage input1,input2,output;
  // Two strings to store the names of the images.
  private String im1,im2;
  // One instance of DisplayThreeSynchronizedImages to display
  // the three images.
  private DisplayThreeSynchronizedImages imagesPanel;

  
 /**
  * Creates the user interface for this JFrame.
  * @param im1 the first input image file name.
  * @param im2 the second input image file name.
  */
  public ArithmeticOps(String im1,String im2)
    {
    super();
    // Store the images' names.
    this.im1 = im1;
    this.im2 = im2;
    // Read the input images.
    input1 = JAI.create("fileload", im1);
    input2 = JAI.create("fileload", im2);
    // Create a temporary output image so we can show something before
    // clicking on the buttons.
    output = createConstFrom(input1);
    // Let's create the graphical user interface. First we create a JPanel
    // with the operations' buttons.
    JPanel buttonPanel = new JPanel();
    buttonPanel.setLayout(new GridLayout(1,4));
    // Let's create the buttons, register their ActionListeners and add them
    // to the JPanel.
    add = new JButton("Add");
    add.addActionListener(this);
    buttonPanel.add(add);
    subtract = new JButton("Subtract");
    subtract.addActionListener(this);
    buttonPanel.add(subtract);
    multiply = new JButton("Multiply");
    multiply.addActionListener(this);
    buttonPanel.add(multiply);
    divide = new JButton("Divide");
    divide.addActionListener(this);
    buttonPanel.add(divide);
    // We'll create an instance of DisplayThreeSynchronizedImages to display the input
    // and output images.
    imagesPanel = new DisplayThreeSynchronizedImages(input1,input2,output);
    // Get the Container for this JFrame.
    Container cp = getContentPane();
    cp.setLayout(new BorderLayout());
    // Add the panel with the buttons and the panel with the images to the
    // container.
    cp.add(imagesPanel,BorderLayout.CENTER);
    cp.add(buttonPanel,BorderLayout.SOUTH);
    // Set the closing operation so the application is finished.
    setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    pack(); // adjust the frame size using preferred dimensions.
    setVisible(true); // show the frame..
    }

 /**
  * Create an image with the same dimensions as the image passed as an
  * argument but with a constant value.
  * @param ip an input PlanarImage
  * @return an output PlanarImage with the same dimensions and bands as the
  *         input one.
  */
  public PlanarImage createConstFrom(PlanarImage pi)
    {
    // Create an array of Bytes with length equal to the number of bands of
    // the input image.
    Byte[] bandValues = new Byte[pi.getNumBands()];
    // Fill the array with a constant value.
    for(int band=0;band<pi.getNumBands();band++)
      bandValues[band] = new Byte((byte)127);
    // Create a ParameterBlock with the parameters for the creation of the
    // output image.
    ParameterBlock pb = new ParameterBlock();
    pb.add((float)pi.getWidth()); // the input image dimensions
    pb.add((float)pi.getHeight());
    pb.add(bandValues);
    // Create and return the constant image.
    return JAI.create("constant", pb, null);
    }

 /**
  * This method will be called when an action event occur (such as the ones
  * generated when an user clicks on the application's buttons).
  * @param e the action event.
  */
  public void actionPerformed(ActionEvent e)
    {
    // Create a ParameterBlock for the images' operation.
    ParameterBlock pb = new ParameterBlock();
    // Add the input images to the ParameterBlock.
    pb.addSource(input1);
    pb.addSource(input2);
    // Run the operation (depending on the source of the event, i.e. the
    // button that was clicked)
    if (e.getSource() == add)
      {
      setTitle("Sum of images "+im1+" and "+im2);
      output = JAI.create("add", pb);
      }
    else if (e.getSource() == subtract)
      {
      setTitle("Image "+im1+" minus image "+im2);
      output = JAI.create("subtract", pb);
      }
    else if (e.getSource() == multiply)
      {
      setTitle("Multiplication of images "+im1+" and "+im2);
      output = JAI.create("multiply", pb);
      }
    else if (e.getSource() == divide)
      {
      setTitle("Sum of images "+im1+" and "+im2);
      output = JAI.create("divide", pb);
      }
    // Change the third image on the DisplayThreeSynchronizedImages.
    imagesPanel.setImage3(output);
    }

 /**
  * The application entry point.
  * @param args the command line arguments.
  */
  public static void main(String[] args)
    {
    // We need two arguments: the input images' names.
    if (args.length != 2)
      {
      System.err.println("Usage: java operators.ArithmeticOps image1 image2");
      System.exit(0);
      }
    new ArithmeticOps(args[0],args[1]);
    }
 
  } // end class