/*
 * Created on Jun 30, 2005
 * @author Rafael Santos (rafael.santos@lac.inpe.br)
 * 
 * Part of the Java Advanced Imaging Stuff site
 * (http://www.lac.inpe.br/~rafael.santos/Java/JAI)
 * 
 * STATUS: Complete.
 * 
 * Redistribution and usage conditions must be done under the
 * Creative Commons license:
 * English: http://creativecommons.org/licenses/by-nc-sa/2.0/br/deed.en
 * Portuguese: http://creativecommons.org/licenses/by-nc-sa/2.0/br/deed.pt
 * More information on design and applications are on the projects' page
 * (http://www.lac.inpe.br/~rafael.santos/Java/JAI).
 */
package operators;

import java.awt.image.renderable.ParameterBlock;
import javax.media.jai.JAI;
import javax.media.jai.PlanarImage;
import javax.swing.JFrame;
import javax.swing.JScrollPane;
import com.sun.media.jai.widget.DisplayJAI;

/**
 * This class shows how to use the bandcombine operator, which will be applied on a
 * RGB image generating two different gray-level images and a color image which 
 * is the result of a linear combination of the original bands.
 */
public class BandCombine
  {
 /**
  * The application entry point. We will need one argument, the image file name.
  * @param args the command-line arguments
  */
  public static void main(String[] args)
    {
    // We need exactly one argument.
    if (args.length != 1)
      {
      System.err.println("Usage: java operators.BandCombine inputImage");
      System.exit(0);
      }
    // Read the input image.
    PlanarImage input = JAI.create("fileload", args[0]);
    // How many bands we have?
    int numBands = input.getNumBands();
    if (numBands != 3)
      {
      System.err.println("Sorry, input image must have three bands.");
      System.exit(0);
      }
    // Create a gray-level image with the average value of the three bands.
    double[][] matrix1 = {{ 1./3, 1./3, 1./3, 0 }};
    ParameterBlock pb1 = new ParameterBlock();
    pb1.addSource(input);
    pb1.add(matrix1);
    PlanarImage gray1 = JAI.create("bandcombine", pb1, null);
    // Display it.
    JFrame frame1 = new JFrame();
    frame1.setTitle("Gray-level by averaging");
    // Add to the JFrame's ContentPane an instance of DisplayJAI with the processed image.
    frame1.getContentPane().add(new JScrollPane(new DisplayJAI(gray1)));
    // Set the closing operation so the application is finished.
    frame1.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    frame1.pack(); // adjust the frame size using preferred dimensions.
    frame1.setVisible(true); // show the frame.
    // Create a gray-level image with the weighted average of the three bands.
    double[][] matrix2 = {{ 0.114, 0.587, 0.299, 0 }};
    ParameterBlock pb2 = new ParameterBlock();
    pb2.addSource(input);
    pb2.add(matrix2);
    PlanarImage gray2 = JAI.create("bandcombine", pb2, null);
    // Display it.
    JFrame frame2 = new JFrame();
    frame2.setTitle("Gray-level by weighted averaging");
    // Add to the JFrame's ContentPane an instance of DisplayJAI with the processed image.
    frame2.getContentPane().add(new JScrollPane(new DisplayJAI(gray2)));
    // Set the closing operation so the application is finished.
    frame2.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    frame2.pack(); // adjust the frame size using preferred dimensions.
    frame2.setVisible(true); // show the frame.
    // Just as an example, a weird 3-band to 3-band transform:
    // newR = 255-R; (inverted)
    // newG = (G+B)/2;
    // newB = 127+B/2;
    double[][] matrix3 = {
          { -1.000, 0.000, 0.000, 255 },
          {  0.000, 0.500, 0.500, 0 },
          {  0.000, 0.000, 0.500, 127 }};
    ParameterBlock pb3 = new ParameterBlock();
    pb3.addSource(input);
    pb3.add(matrix3);
    PlanarImage color = JAI.create("bandcombine", pb3, null);
    // Display it.
    JFrame frame3 = new JFrame();
    frame3.setTitle("3-band to 3-band mapping");
    // Add to the JFrame's ContentPane an instance of DisplayJAI with the processed image.
    frame3.getContentPane().add(new JScrollPane(new DisplayJAI(color)));
    // Set the closing operation so the application is finished.
    frame3.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    frame3.pack(); // adjust the frame size using preferred dimensions.
    frame3.setVisible(true); // show the frame.
    }

  }