/*
 * Created on May 20, 2005
 * @author Rafael Santos (rafael.santos@lac.inpe.br)
 * 
 * Part of the Java Advanced Imaging Stuff site
 * (http://www.lac.inpe.br/~rafael.santos/Java/JAI)
 * 
 * STATUS: Complete.
 * 
 * Redistribution and usage conditions must be done under the
 * Creative Commons license:
 * English: http://creativecommons.org/licenses/by-nc-sa/2.0/br/deed.en
 * Portuguese: http://creativecommons.org/licenses/by-nc-sa/2.0/br/deed.pt
 * More information on design and applications are on the projects' page
 * (http://www.lac.inpe.br/~rafael.santos/Java/JAI).
 */
package operators;

import javax.media.jai.JAI;
import javax.media.jai.ParameterBlockJAI;
import javax.media.jai.PlanarImage;

/**
 * This class demonstrates the use of the bandmerge operator.
 */
public class BandMerge
  {
 /**
  * The application entry point.
  * @param args the command line arguments. The first arguments are the
  * name of the separated bands, the last argument is the name of the file
  * that will be created with the combined bands.
  */
  public static void main(String[] args)
    {
    // We need at least two arguments.
    if (args.length < 2)
      {
      System.err.println("Usage: java operators.BandMerge image1 image2 "+
                         "... outputImage");
      System.exit(0);
      }
    // Read the input images.
    PlanarImage[] inputs = new PlanarImage[args.length-1];
    for(int im=0;im<args.length-1;im++)
      inputs[im] = JAI.create("fileload", args[im]);
    // Create a ParameterBlockJAI and set the input images as its sources
    ParameterBlockJAI pbjai = new ParameterBlockJAI("bandmerge");
    // Set the sources to be the input images.
    for(int im=0;im<args.length-1;im++)
      pbjai.setSource(inputs[im], im);
    // Create the merged image.
    PlanarImage result = JAI.create("bandmerge",pbjai,null);
    // Store it as a TIFF since it supports multiband images.
    JAI.create("filestore",result,args[args.length-1],"TIFF");
    } // end main 
  } // end class