/*
 * Created on June 13, 2005
 * @author Rafael Santos (rafael.santos@lac.inpe.br)
 * 
 * Part of the Java Advanced Imaging Stuff site
 * (http://www.lac.inpe.br/~rafael.santos/Java/JAI)
 * 
 * STATUS: Complete.
 * 
 * Redistribution and usage conditions must be done under the
 * Creative Commons license:
 * English: http://creativecommons.org/licenses/by-nc-sa/2.0/br/deed.en
 * Portuguese: http://creativecommons.org/licenses/by-nc-sa/2.0/br/deed.pt
 * More information on design and applications are on the projects' page
 * (http://www.lac.inpe.br/~rafael.santos/Java/JAI).
 */
package operators;

import java.awt.BorderLayout;
import java.awt.Font;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.image.renderable.ParameterBlock;
import javax.media.jai.JAI;
import javax.media.jai.KernelJAI;
import javax.media.jai.PlanarImage;
import javax.swing.ButtonGroup;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import display.DisplayTwoSynchronizedImages;

/**
 * This class demonstrates the use of the erode and dilate (mathematical morphology) operators.
 */
public class MMorph extends JFrame implements ActionListener
  {
  // The original image and the one that will be processed.
  private PlanarImage image,processedImage;
  // The display instance.
  private DisplayTwoSynchronizedImages display;
  // Some graphical user interface components.
  private JRadioButton erode,dilate;
  private Font font = new Font("default",0,28);
  private JButton kern1Button,kern2Button,kern3Button,kern4Button,kern5Button;
  // The kernels for the operations. 
  private static float[] kernel1Matrix = { 127, 127, 127, 127, 127, 127, 127, 
                                           127, 127, 127, 127, 127, 127, 127, 
                                           127, 127, 127, 127, 127, 127, 127, 
                                           127, 127, 127, 127, 127, 127, 127, 
                                           127, 127, 127, 127, 127, 127, 127, 
                                           127, 127, 127, 127, 127, 127, 127, 
                                           127, 127, 127, 127, 127, 127, 127};
  private static float[] kernel2Matrix = {   0,   0,   0, 127,   0,   0,   0, 
                                             0,   0,   0, 127,   0,   0,   0, 
                                             0,   0,   0, 127,   0,   0,   0, 
                                             0,   0,   0, 127,   0,   0,   0, 
                                             0,   0,   0, 127,   0,   0,   0, 
                                             0,   0,   0, 127,   0,   0,   0, 
                                             0,   0,   0, 127,   0,   0,   0 };
  private static float[] kernel3Matrix = {   0,   0,   0,   0,   0,   0,   0, 
                                             0,   0,   0,   0,   0,   0,   0, 
                                             0,   0,   0,   0,   0,   0,   0, 
                                           127, 127, 127, 127, 127, 127, 127, 
                                             0,   0,   0,   0,   0,   0,   0, 
                                             0,   0,   0,   0,   0,   0,   0, 
                                             0,   0,   0,   0,   0,   0,   0 };
  private static float[] kernel4Matrix = { 127, 127, 127, 127, 127, 127, 127, 
                                           127,   0,   0,   0,   0,   0, 127, 
                                           127,   0,   0,   0,   0,   0, 127, 
                                           127,   0,   0,   0,   0,   0, 127, 
                                           127,   0,   0,   0,   0,   0, 127, 
                                           127,   0,   0,   0,   0,   0, 127, 
                                           127, 127, 127, 127, 127, 127, 127 };
  private static float[] kernel5Matrix = { 127,   0,   0,   0,   0,   0, 127, 
                                             0, 127,   0,   0,   0, 127,   0, 
                                             0,   0, 127,   0, 127,   0,   0, 
                                             0,   0,   0, 127,   0,   0,   0, 
                                             0,   0, 127,   0, 127,   0,   0, 
                                             0, 127,   0,   0,   0, 127,   0, 
                                           127,   0,   0,   0,   0,   0, 127 };
  private KernelJAI kern1 = new KernelJAI(7,7,3,3,kernel1Matrix);
  private KernelJAI kern2 = new KernelJAI(7,7,3,3,kernel2Matrix);
  private KernelJAI kern3 = new KernelJAI(7,7,3,3,kernel3Matrix);
  private KernelJAI kern4 = new KernelJAI(7,7,3,3,kernel4Matrix);
  private KernelJAI kern5 = new KernelJAI(7,7,3,3,kernel5Matrix);
  
 /**
  * The constructor for this class, which will set its user interface.
  */
  public MMorph(PlanarImage i)
    {
    super("Mathematical morphology operators");
    image = i;
    // We need a temporary image for the display.
    ParameterBlock p = new ParameterBlock();
    p.add((float)image.getWidth()); // float width and height?
    p.add((float)image.getHeight());
    Byte[] bandValues = new Byte[image.getNumBands()];  
    for(int band=0;band<bandValues.length;band++) bandValues[band] = new Byte((byte)127);  
    p.add(bandValues);
    processedImage = JAI.create("constant",p,null);
    // Create an instance of DisplayTwoSynchronizedImages to hold both images.
    display = new DisplayTwoSynchronizedImages(image,processedImage);
    getContentPane().add(display);
    // Let's create the UI. First a panel for the controls.
    JPanel controlPanel = new JPanel(new GridLayout(1,7));
    // Then, two radio buttons for the operations.
    erode = new JRadioButton("Erode"); erode.setFont(font); erode.setSelected(true); 
    dilate = new JRadioButton("Dilate"); dilate.setFont(font);  
    ButtonGroup group = new ButtonGroup();
    group.add(erode); group.add(dilate);
    controlPanel.add(erode); controlPanel.add(dilate);
    // Five icon buttons with the operators.
    kern1Button = new JButton(new ImageIcon(Class.class.getResource("/operators/icons/kernel1.png")));
    controlPanel.add(kern1Button); kern1Button.addActionListener(this);
    kern2Button = new JButton(new ImageIcon(Class.class.getResource("/operators/icons/kernel2.png")));
    controlPanel.add(kern2Button); kern2Button.addActionListener(this);
    kern3Button = new JButton(new ImageIcon(Class.class.getResource("/operators/icons/kernel3.png")));
    controlPanel.add(kern3Button); kern3Button.addActionListener(this);
    kern4Button = new JButton(new ImageIcon(Class.class.getResource("/operators/icons/kernel4.png")));
    controlPanel.add(kern4Button); kern4Button.addActionListener(this);
    kern5Button = new JButton(new ImageIcon(Class.class.getResource("/operators/icons/kernel5.png")));
    controlPanel.add(kern5Button); kern5Button.addActionListener(this);
    // Add the control panel to the UI.
    getContentPane().add(controlPanel,BorderLayout.SOUTH);
    // Set the closing operation so the application is finished.
    setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    pack(); // adjust the frame size using preferred dimensions.
    setVisible(true); // show the frame.
    }

 /**
  * This method will be called when the user selects a button.
  */
  public void actionPerformed(ActionEvent e)
    {
    ParameterBlock p = new ParameterBlock();
    p.addSource(image);
    if (e.getSource() == kern1Button) p.add(kern1);
    else if (e.getSource() == kern2Button) p.add(kern2);
    else if (e.getSource() == kern3Button) p.add(kern3);
    else if (e.getSource() == kern4Button) p.add(kern4);
    else if (e.getSource() == kern5Button) p.add(kern5);
    if (erode.isSelected()) processedImage = JAI.create("erode",p,null);
    else processedImage = JAI.create("dilate",p,null);
    display.setImage2(processedImage);
    }
  
 /**
  * The application entry point.
  * @param args the command line arguments.
  */
  public static void main(String[] args)
    {
    // We need one argument: the image filename.
    if (args.length != 1)
      {
      System.err.println("Usage: java operators.MMorph image");
      System.exit(0);
      }
    // Read the image.
    PlanarImage input = JAI.create("fileload", args[0]);
    // Create an instance of this class to process and display the results.
    new MMorph(input);
    }

  }