/*
 * Created on May 22, 2005
 * @author Rafael Santos (rafael.santos@lac.inpe.br)
 * 
 * Part of the Java Advanced Imaging Stuff site
 * (http://www.lac.inpe.br/~rafael.santos/Java/JAI)
 * 
 * STATUS: Complete.
 * 
 * Redistribution and usage conditions must be done under the
 * Creative Commons license:
 * English: http://creativecommons.org/licenses/by-nc-sa/2.0/br/deed.en
 * Portuguese: http://creativecommons.org/licenses/by-nc-sa/2.0/br/deed.pt
 * More information on design and applications are on the projects' page
 * (http://www.lac.inpe.br/~rafael.santos/Java/JAI).
 */
package operators;

import java.awt.image.renderable.ParameterBlock;
import javax.media.jai.JAI;
import javax.media.jai.KernelJAI;
import javax.media.jai.PlanarImage;
import javax.swing.JFrame;
import display.DisplayTwoSynchronizedImages;

/**
 * This class demonstrates the use of the erode and dilate operators to perform
 * opening on an image. It should work with binary images only.
 */
public class Open
  { 
 /**
  * The application entry point.
  * @param args the command line arguments.
  */
  public static void main(String[] args)
    {
    // We need one argument - the image file name.
    if (args.length != 1)
      {
      System.err.println("Usage: java operators.Open image");
      System.exit(0);
      }
    // Read the image.
    PlanarImage input = JAI.create("fileload", args[0]);
    // The kernels for the operations. 
    float[] kernelMatrix = {   0,   0,   0,   0,   0,   0,   0, 
                               0,   1,   1,   1,   1,   1,   0, 
                               0,   1,   1,   1,   1,   1,   0, 
                               0,   1,   1,   1,   1,   1,   0, 
                               0,   1,   1,   1,   1,   1,   0, 
                               0,   1,   1,   1,   1,   1,   0, 
                               0,   0,   0,   0,   0,   0,   0};
    // Create the kernel using the array.
    KernelJAI kernel = new KernelJAI(7,7,kernelMatrix);
    // Create a ParameterBlock with that kernel and image.
    ParameterBlock p = new ParameterBlock();
    p.addSource(input);
    p.add(kernel);
    // Erode the image.
    PlanarImage output = JAI.create("erode",p,null);
    // Now dilate the image with the same kernel.
    p = new ParameterBlock();
    p.addSource(output);
    p.add(kernel);
    output = JAI.create("dilate",p,null);
    // Create a JFrame for displaying the results.
    JFrame frame = new JFrame();
    frame.setTitle("Opening of the image "+args[0]);
    // Add to the JFrame's ContentPane an instance of
    // DisplayTwoSynchronizedImages, which will contain the original and
    // processed image.
    frame.getContentPane().add(new DisplayTwoSynchronizedImages(input,output));
    // Set the closing operation so the application is finished.
    frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    frame.pack(); // adjust the frame size using preferred dimensions.
    frame.setVisible(true); // show the frame.
    }

  } // end class