/*
 * Created on Nov 19, 2005
 * @author Rafael Santos (rafael.santos@lac.inpe.br)
 * 
 * Part of the Java Advanced Imaging Stuff site
 * (http://www.lac.inpe.br/~rafael.santos/Java/JAI)
 * 
 * STATUS:
 * - Still alpha / working for some tasks or parameters / fully functional
 * - Needs documentation on the source code
 * - Needs documentation on the page
 * 
 * 
 * Redistribution and usage conditions must be done under the
 * Creative Commons license:
 * English: http://creativecommons.org/licenses/by-nc-sa/2.0/br/deed.en
 * Portuguese: http://creativecommons.org/licenses/by-nc-sa/2.0/br/deed.pt
 * More information on design and applications are on the projects' page
 * (http://www.lac.inpe.br/~rafael.santos/Java/JAI).
 */
package operators;

import java.awt.Dimension;
import java.awt.RenderingHints;
import java.awt.Transparency;
import java.awt.image.ColorModel;
import java.awt.image.ComponentColorModel;
import java.awt.image.DataBuffer;
import java.awt.image.RenderedImage;
import java.awt.image.renderable.ParameterBlock;
import javax.media.jai.IHSColorSpace;
import javax.media.jai.ImageLayout;
import javax.media.jai.JAI;
import javax.swing.JFrame;
import display.DisplayFourSynchronizedImages;

/**
 * This class demonstrates the colorconvert operator, converting a RGB image
 * to IHS, making some manipulations on the I and S bands and reconverting to
 * RGB for displaying.
 * Part of this code is based on a message from Madhu V Reddy on the 
 * Java Advanced Imaging API discussion group:
 * http://archives.java.sun.com/cgi-bin/wa?A2=ind0308&L=jai-interest&F=&S=&P=623
 */
public class RGBtoIHS
  {
 /**
  * @param args the command-line arguments (the original image must be speficied)
  */
  public static void main(String[] args)
    {
    // Load the input image from a file. 
    RenderedImage srcImage = JAI.create("fileload", args[0]);
    // See https://jai-core.dev.java.net/issues/show_bug.cgi?id=30 for 
    // an explanation about the code below.
    JAI.setDefaultTileSize(new Dimension(srcImage.getTileWidth(),
                                         srcImage.getTileHeight()));
    // Create a ColorModel to convert the image to IHS.
    IHSColorSpace ihs = IHSColorSpace.getInstance();
    ColorModel ihsColorModel = 
       new ComponentColorModel(ihs,
                               new int []{8,8,8},
                               false,false,
                               Transparency.OPAQUE,
                               DataBuffer.TYPE_BYTE) ;
    // Create a ParameterBlock for the conversion.
    ParameterBlock pb = new ParameterBlock();
    pb.addSource(srcImage);
    pb.add(ihsColorModel);
    // Do the conversion.
    RenderedImage ihsImage  = JAI.create("colorconvert", pb);
    // Extract the I,H,S bands. We'll do it in an array of RendededImages.
    RenderedImage[] bands = new RenderedImage[3];
    for(int band=0;band<3;band++)
      {
      pb = new ParameterBlock();
      pb.addSource(ihsImage);
      pb.add(new int[]{band});
      bands[band] = JAI.create("bandselect",pb);
      }
    // We would like to display the H band with colors, e.g. with maximum
    // intensity and saturation. Let's create some artificial bands, join them
    // and convert the results.
    pb = new ParameterBlock();
    pb.add((float)srcImage.getWidth());
    pb.add((float)srcImage.getHeight());
    pb.add(new Byte[]{(byte)255});
    RenderedImage newIntensity = JAI.create("constant",pb);
    pb = new ParameterBlock();
    pb.add((float)srcImage.getWidth());
    pb.add((float)srcImage.getHeight());
    pb.add(new Byte[]{(byte)255});
    RenderedImage newSaturation = JAI.create("constant",pb);
    // Let's merge the modified IHS image. The message on the mailing list
    // (see comments for this class) mentioned that it is required to pass
    // a RenderingHing with a ImageLayout with the IHS color model.
    ImageLayout imageLayout = new ImageLayout();
    imageLayout.setColorModel(ihsColorModel);
    imageLayout.setSampleModel(ihsImage.getSampleModel());
    RenderingHints rendHints = 
      new RenderingHints(JAI.KEY_IMAGE_LAYOUT,imageLayout);
    // Create the parameterblock for the merging of the bands     
    pb = new ParameterBlock();
    pb.addSource(newIntensity);
    pb.addSource(bands[1]);
    pb.addSource(newSaturation);
    RenderedImage modifiedIhsImage = JAI.create("bandmerge", pb, rendHints);
    // Now we convert the modified IHS image back to RGB for displaying. 
    pb = new ParameterBlock();
    pb.addSource(modifiedIhsImage);
    pb.add(srcImage.getColorModel()); // RGB color model!
    RenderedImage finalImage = JAI.create("colorconvert", pb);
    // OK, we're ready to display those images.
    // Create a JFrame for displaying the results.
    JFrame frame = new JFrame();
    frame.setTitle("RGB-IHS conversion of "+args[0]);
    // Add to the JFrame's ContentPane an instance of DisplayFourSynchronizedImages, 
    // which will contain the original image and IHS bands.
    DisplayFourSynchronizedImages disp = 
      new DisplayFourSynchronizedImages(srcImage,bands[0],bands[2],finalImage);  
    frame.getContentPane().add(disp);
    // Set the closing operation so the application is finished.
    frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    frame.pack(); // adjust the frame size using preferred dimensions.
    frame.setVisible(true); // show the frame.
    }
  
  }