/*
 * Created on May 22, 2005
 * @author Rafael Santos (rafael.santos@lac.inpe.br)
 * 
 * Part of the Java Advanced Imaging Stuff site
 * (http://www.lac.inpe.br/~rafael.santos/Java/JAI)
 * 
 * STATUS: Complete.
 * 
 * Redistribution and usage conditions must be done under the
 * Creative Commons license:
 * English: http://creativecommons.org/licenses/by-nc-sa/2.0/br/deed.en
 * Portuguese: http://creativecommons.org/licenses/by-nc-sa/2.0/br/deed.pt
 * More information on design and applications are on the projects' page
 * (http://www.lac.inpe.br/~rafael.santos/Java/JAI).
 */
package operators;

import javax.media.jai.JAI;
import javax.media.jai.KernelJAI;
import javax.media.jai.PlanarImage;
import javax.swing.JFrame;
import display.DisplayTwoSynchronizedImages;

/**
 * This class demonstrates the use of the convolve operator for smoothing an image.
 */
public class Smooth
  {
 /**
  * The application entry point.
  * @param args the command line arguments.
  */
  public static void main(String[] args)
    {
    // We need two arguments: the image filename and the kernel size.
    // A square kernel will be used.
    if (args.length != 2)
      {
      System.err.println("Usage: java operators.Smooth image kernelsize");
      System.exit(0);
      }
    // Get the kernel size (converts to integer).
    int kernelSize = Integer.parseInt(args[1]);
    // Create an array with enough positions for the kernel.
    float[] kernelMatrix = new float[kernelSize*kernelSize];
    // Fill the array with the data for the kernel. The sum of all array
    // positions will be 1.0.
    for(int k=0;k<kernelMatrix.length;k++)
      kernelMatrix[k] = 1.0f/(kernelSize*kernelSize);
    // Read the image.
    PlanarImage input = JAI.create("fileload", args[0]);
    // Create the kernel using the array.
    KernelJAI kernel = new KernelJAI(kernelSize,kernelSize,kernelMatrix);
    // Run the convolve operator, creating the output image.
    PlanarImage output = JAI.create("convolve", input, kernel);
    // Create a JFrame for displaying the results.
    JFrame frame = new JFrame();
    frame.setTitle("Smooth image "+args[0]+" with kernel "+args[1]+"x"+args[1]);
    // Add to the JFrame's ContentPane an instance of
    // DisplayTwoSynchronizedImages, which will contain the origin and
    // processed image.
    frame.getContentPane().add(new DisplayTwoSynchronizedImages(input,output));
    // Set the closing operation so the application is finished.
    frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    frame.pack(); // adjust the frame size using preferred dimensions.
    frame.setVisible(true); // show the frame.
    }

  } // end class