/*
 * Created on 6 juin 2005 by richet
 */
package org.math.array;

import org.math.array.linearalgebra.*;

public class LinearAlgebra extends DoubleArray {

    // linear algebra methods

    public static double[][] transpose(double[][] M) {
        double[][] tM = new double[M[0].length][M.length];
        for (int i = 0; i < tM.length; i++)
            for (int j = 0; j < tM[0].length; j++)
                tM[i][j] = M[j][i];
        return tM;
    }

    public static double[] minus(double[] v1, double[] v2) {
        double[] array = new double[v1.length];
        for (int i = 0; i < v1.length; i++)
            array[i] = v1[i] - v2[i];
        return array;
    }

    public static double[] minus(double[] v1, double v) {
        double[] array = new double[v1.length];
        for (int i = 0; i < v1.length; i++)
            array[i] = v1[i] - v;
        return array;
    }

    public static double[] minus(double v, double[] v1) {
        double[] array = new double[v1.length];
        for (int i = 0; i < v1.length; i++)
            array[i] = v - v1[i];
        return array;
    }

    public static double[] plus(double[]... v) {
        double[] array = new double[v[0].length];
        for (int j = 0; j < v.length; j++)
            for (int i = 0; i < v[j].length; i++)
                array[i] += v[j][i];
        return array;
    }

    public static double[] plus(double[] v1, double v) {
        double[] array = new double[v1.length];
        for (int i = 0; i < v1.length; i++)
            array[i] = v1[i] + v;
        return array;
    }

    public static double[] times(double[]... v) {
        double[] array = one(v[0].length);
        for (int j = 0; j < v.length; j++)
            for (int i = 0; i < v[j].length; i++)
                array[i] *= v[j][i];
        return array;
    }

    public static double[] divide(double[] v1, double[] v2) {
        checkLength(v1, v2.length);
        double[] array = new double[v1.length];
        for (int i = 0; i < v1.length; i++)
            array[i] = v1[i] / v2[i];
        return array;
    }

    public static double[] times(double[] v1, double v) {
        double[] array = new double[v1.length];
        for (int i = 0; i < v1.length; i++)
            array[i] = v1[i] * v;
        return array;
    }

    public static double[] divide(double[] v1, double v) {
        double[] array = new double[v1.length];
        for (int i = 0; i < v1.length; i++)
            array[i] = v1[i] / v;
        return array;
    }

    public static double[][] times(double[][] v1, double[][] v2) {
        // checkColumnDimension(v1, v2.length);
        // checkRowDimension(v1, v2[0].length);
        double[][] array = new double[v1.length][v2[0].length];
        for (int i = 0; i < array.length; i++)
            for (int j = 0; j < array[i].length; j++) {
                double tmp = 0;
                for (int k = 0; k < v1[0].length; k++)
                    tmp += v1[i][k] * v2[k][j];
                array[i][j] = tmp;
            }
        return array;
    }

    public static EigenvalueDecomposition eigenValueDecomposition(double[][] v) {
        return new EigenvalueDecomposition(v);
    }

    public static QRDecomposition qrDecomposition(double[][] v) {
        return new QRDecomposition(v);
    }

    public static LUDecomposition luDecomposition(double[][] v) {
        return new LUDecomposition(v);
    }

    public static CholeskyDecomposition choleskyDecomposition(double[][] v) {
        return new CholeskyDecomposition(v);
    }

}
