

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.util.Hashtable;
import javax.media.jai.*;
import java.awt.*;
import java.awt.geom.Ellipse2D;
import java.awt.geom.Rectangle2D;
import javax.media.jai.iterator.RandomIter;
import javax.media.jai.iterator.RandomIterFactory;
import java.awt.image.renderable.ParameterBlock;

import static org.math.io.files.ASCIIFile.*;
import static org.math.io.parser.ArrayString.*;
import org.math.array.*;

import java.io.*; 
import jhplot.*;

/**
 * This applications shows a rather complex usage of the KMeansImageClustering class, 
 * creating a user interface that allows the user to select some clustering 
 * parameters and rerun the cluster task. The original and clustered image are 
 * shown, side-by-side and synchronized. Input data is read from a file and at 
 * the end of the clustering process the data is saved in another file.
 */
public class testMain 
  {
  // The input and output images.
  private PlanarImage input,output;
  private KMeansImageClustering clusterer;
  // The output image file name.
  private String ofile="output";
  private KMeansClusterDescriptor[] results;
 
 /**
  * The constructor for the application, which will set its user interface.
  * @param ifile the input file name.
  * @param ofile the output file name.
  */
  public testMain(String ifile, String what) { 
    // Read the input image from the file.
    input = JAI.create("fileload", ifile);
    this.ofile = ofile;

  //  String type="photo"; // set true to analyse photo 
    String type="plot";
    if (ifile.startsWith("images1/")) type="photo";
    if (ifile.startsWith("images2/")) type="plot";



    boolean FAST=true;
    char iMethod='T';
    int maxIter = 10000;
    int epsilon = 5;
    int Ntot=12; 
    int NtotClus=10; 

   
    double[][] x = new double[NtotClus][4];
    double[][] y = new double[NtotClus][4];
    String[] legend=new String[4]; 
    legend[0]="XY comp";
    legend[1]="COL comp";
    legend[2]="XY size";
    legend[3]="COL size";


    double[][] out1 = new double[NtotClus][9];
    double[][] out2 = new double[NtotClus][9];
    double[] out1COL = new double[64];
    double[] out2COL = new double[64];

   String xmlout=ifile+".xml"; 
   StopWatch sw= new StopWatch(); 
   sw.start();

      boolean iREAD=false;
      if (what.equals("read"))  {
           System.out.println("Reading sampling1.txt and sampling2.txt");
           out2=readDoubleArray(new File("sampling1.txt"));
           out2COL=readDouble1DArray(new File("sampling2.txt"));
            iREAD=true;
       }

       
     // Create the clustering task and starts it.
      int numBands = input.getSampleModel().getNumBands();
      double eSimilarity=(double)numBands;
      int w = input.getWidth();
      int h = input.getHeight();
      int[] aPixel = new int[numBands];
      RandomIter iterator = RandomIterFactory.create(input,null);

       double x1, ex1;
       int[]    ndf  = new int[10];
       double[] chi2 = new double[10];
       double[] p = new double[10];
       double[][] pSoFar  = new double [10][NtotClus];


    PlanarImage image = JAI.create("fileload", ifile);
    ParameterBlock pb = new ParameterBlock();
    pb.addSource(image);
    pb.add(null); // The ROI.

// color analysis
    pb.add(1); pb.add(1); // Sampling
    pb.add(new int[]{64}); // Bins
    pb.add(new double[]{0});  pb.add(new double[]{256}); // Range for inclusion
    // Creates the histogram.
    PlanarImage temp = JAI.create("histogram", pb);
    javax.media.jai.Histogram hh = (javax.media.jai.Histogram)temp.getProperty("histogram");
    System.out.println("getNumBands()="+hh.getNumBands());
    double[] mean = hh.getMean();
    double[] rms =  hh.getStandardDeviation();
    for (int i=0; i<hh.getNumBands(); i++){
     System.out.println(i+" getMean()="+mean[i]);
     System.out.println(i+" getStandardDeviation()="+rms[i]);
     System.out.println(i+" getHighValue()="+hh.getHighValue(i));
    }

    int binWidth = numBands;
     // Calculate the components dimensions.
    double width = hh.getNumBins(0)*binWidth;
    // Get the histogram data.
    int  [] counts = hh.getBins(0);
    // Get the max and min counts.
    int  maxCount = Integer.MIN_VALUE;
    for(int c=0;c<counts.length;c++) maxCount = Math.max(maxCount,counts[c]);
     H1D hhh = new H1D("NEW COLORS",hh.getNumBins(0), 0,256);
     hhh.setHights(counts);
     double area1=hhh.area();
     // hhh.normilize();
     // hhh.show();
     out1COL=hhh.getHights();

       if (iREAD==false) {
           hhh.toFile("sampling2.txt");
           System.out.println("Writing  sampling2.txt");
       } else {

          // normalize 
          H1D hhh2 = new H1D("OLD COLORS",out2COL.length, 0,256);
          hhh2.setHights(out2COL);
         // hhh2.normilize();
         // hhh2.show();
          double area2=hhh2.area();

          ndf[0]=0;
          chi2[0]=0; 
          for (int i=0; i<out1COL.length; i++) {
          if (out1COL[i]  >=0 && out2COL[i]>=0) {
             double dw=(out1COL[i]/area1)  - (out2COL[i]/area2);
             //double ss=0.5*(out1COL[i]/area1)  + 0.5*(out2COL[i]/area2);
             // System.out.println(out1COL[i]);
             // assume we can separate color withing 5 units
             // double eex1= 2/area1;
             // double eex2= 2./area2;
             //  double ee = out2COL[i]/area2; 
              // double ee=Math.sqrt(eex1*eex1 + eex2*eex2);
              chi2[0] +=( dw*dw/ 0.0001 );
              ndf[0]++;  
         }
         } 
          // p0= Math.sqrt(xx1)/(pc);
           // p0=StatTools.chi2Df(chi2,ndf); 
           System.out.println("\n==> chi2="+chi2[0]+ " Ndf="+ndf[0]);
           p[0]=chi2ndf(chi2[0],ndf[0]);
           System.out.println("\n==> ColorSpectrum chi2/ndf="+p[0]);
           for (int i=1; i<NtotClus; i++) pSoFar[0][i]=p[0];



           if (FAST==true &&  p[0]<0.00001) {
             System.out.println("\n==> Identical figures!");
             String sfile=ofile+".d";
             sw.stop();
             new Decision(type,chi2,ndf,pSoFar,ifile,Double.toString(sw.getTime()/1000));
             Decision.print(chi2,ndf);
             System.exit(0);
           }


        
      } // end read


      for (int ICLus=2; ICLus<NtotClus;  ICLus++) {

      System.out.println("\n==> No of clusters for analysis="+ICLus);

      int nClusters =ICLus; 
      float[][] iniCenters = new float[nClusters][numBands];
      float[][][] Centers = new float[nClusters][numBands][Ntot];




       double[] quality = new double[Ntot];
       double[] ct1 = new double[Ntot];
       double[] ct2 = new double[Ntot];
       double[] ct3 = new double[Ntot];
       double[] ct4 = new double[Ntot];
       double std1, std2, std3, std4;

       // keep invalid indexis 
       int[]    invalid = new int[Ntot];
       int NN=0;

       for (int ILoop=0; ILoop<Ntot;  ILoop++) {
       // initial
       for(int cluster=0;cluster<nClusters;cluster++) {
        int rx = (int)(Math.random()*w);
        int ry = (int)(Math.random()*h);
        iterator.getPixel(rx,ry,aPixel);
        for(int band=0;band<numBands;band++) { 
                    iniCenters[cluster][band] = aPixel[band];
                    Centers[cluster][band][ILoop]=aPixel[band];;
                    }
        }
       // end initail


       clusterer = new KMeansImageClustering(input,nClusters,maxIter,
                                             epsilon, iMethod, iniCenters);
       clusterer.run();

       double t1=clusterer.getQualityXY();
       double t2=clusterer.getQualityCol();
       double t3=clusterer.getDistanceXY();
       double t4=clusterer.getDistanceCol();

       // check for NAN
       invalid[ILoop]=0;
       if (t1 !=  t1 || t2 !=  t2) {
           invalid[ILoop]=1;
           NN++;
       } 
       if (t1 !=  t1) t1=-9999.0;
       if (t2 !=  t2) t2=-9999.0;
       if (t3 !=  t3) t3=-9999.0;
       if (t4 !=  t4) t4=-9999.0;

       ct1[ILoop]=t1;
       ct2[ILoop]=t2;
       quality[ILoop]=t1; // t1*t2;
       ct3[ILoop]=t3;
       ct4[ILoop]=t4;

       // System.out.println("\nGlobal XY compactness="+Double.toString(clusterer.getQuality()));
       // System.out.println("Global Color compactness="+Double.toString(clusterer.getQualityCol()));
       
       } // end loop



// stop loop if all are bad
      if (NN == Ntot) {
        System.out.println("EXIT LOOP. ALL ARE BAD");
        break;
     }  




      // remove invalid entries
      System.out.println("Total checked="+Ntot);
     
      double[] xquality = new double[Ntot];
      double[] xct1 = new double[Ntot];
      double[] xct2 = new double[Ntot];
      double[] xct3 = new double[Ntot];
      double[] xct4 = new double[Ntot];


      if (NN>0 && NN < Ntot) { 
      
       int k=0;
       xquality = new double[Ntot-NN];
       xct1 = new double[Ntot-NN];
       xct2 = new double[Ntot-NN];
       xct3 = new double[Ntot-NN];
       xct4 = new double[Ntot-NN];

         for (int i=0; i<Ntot; i++) {
         if (invalid[i] == 1) continue; 
         xct1[k]=ct1[i];
         xct2[k]=ct2[i];
         xct3[k]=ct3[i];
         xct4[k]=ct4[i];
         xquality[k]=quality[i];
         k++;
         }
      } else {
      xquality = new double[Ntot];
      xct1 = new double[Ntot];
      xct2 = new double[Ntot];
      xct3 = new double[Ntot];
      xct4 = new double[Ntot];

         System.arraycopy(ct1, 0, xct1, 0, ct1.length);
         System.arraycopy(ct2, 0, xct2, 0, ct2.length);
         System.arraycopy(ct3, 0, xct3, 0, ct3.length);
         System.arraycopy(ct4, 0, xct4, 0, ct4.length);
         System.arraycopy(quality, 0, xquality, 0, quality.length);
      }


      int ibest = DoubleArray.minIndex( xquality );

      // System.out.println("ct1 =  \n" + printDoubleArray(xct1));
      // System.out.println("ct2 =  \n" + printDoubleArray(xct2));
      // System.out.println("quality =  \n" + printDoubleArray(xquality));
 
      std1 = StatisticSample.stddeviation(xct1);
      std2 = StatisticSample.stddeviation(xct2);
      std3 = StatisticSample.stddeviation(xct3);
      std4 = StatisticSample.stddeviation(xct4);

      if (std1 != std1) std1 =0.1;
      if (std2 != std2) std2 =1;
      if (std3 != std3) std3 =0.01;
      if (std4 != std4) std4 =0.01;

      if (std1 <0.0001) std1 =0.01;
      if (std2 <0.0001) std2 =1;
      if (std3 <0.0001) std3 =0.01;
      if (std4 <0.0001) std4 =0.01;

      System.out.println("Best XY*COL compactness="+Double.toString(  xquality[ibest]));
      System.out.println(" XY  comp="+Double.toString(xct1[ibest])+"+/-"+std1);
      System.out.println(" COL comp="+Double.toString(xct2[ibest])+"+/-"+std2);
      System.out.println(" XY  R="+Double.toString(xct3[ibest])+"+/-"+std3);
      System.out.println(" COL R="+Double.toString(xct4[ibest])+"+/-"+std4);
      System.out.println("      No invalid="+Integer.toString(NN));


       for(int cluster=0;cluster<nClusters;cluster++) {
        for(int band=0;band<numBands;band++)
                    iniCenters[cluster][band] = Centers[cluster][band][ibest];
        }



       // rerun for best
       clusterer = new KMeansImageClustering(input,nClusters,
                       maxIter,epsilon, iMethod, iniCenters);
       clusterer.run();

       double t1=clusterer.getQualityXY();
       double t2=clusterer.getQualityCol();
       double t3=clusterer.getDistanceXY();
       double t4=clusterer.getDistanceCol();

       // check for NAN
       if (t1 !=  t1) t1=-999.0;
       if (t2 !=  t2) t2=-999.0;
       if (t3 !=  t3) t3=-999.0;
       if (t4 !=  t4) t4=-999.0;


      //  System.out.println("\nBest XY compactness="+t1);
      // System.out.println("Best Color compactness="+t2);

       x[ICLus][0]=ICLus;  
       y[ICLus][0]=t1;

       x[ICLus][1]=ICLus;
       y[ICLus][1]=t2;

       x[ICLus][2]=ICLus;
       y[ICLus][2]=t3;

       x[ICLus][3]=ICLus;
       y[ICLus][3]=t4;


       out1[ICLus][0]=(double)ICLus;
       out1[ICLus][1]=t1;
       out1[ICLus][2]=std1;
       out1[ICLus][3]=t2;
       out1[ICLus][4]=std2;
       out1[ICLus][5]=t3;
       out1[ICLus][6]=std3;
       out1[ICLus][7]=t4;
       out1[ICLus][8]=std4;


        // calculate probability
        if (ICLus>1 && iREAD==true) {

        if (out1[ICLus][1] >0 && out2[ICLus][1]>0) {
             x1=out1[ICLus][1]-out2[ICLus][1];
             ex1=out1[ICLus][2]*out1[ICLus][2]+out2[ICLus][2]*out2[ICLus][2];
             chi2[1] +=( x1*x1/ex1 );
             ndf[1]++;
         }

         if (out1[ICLus][3] >0 && out2[ICLus][3]>0) {
             x1=out1[ICLus][3]-out2[ICLus][3];
             ex1=out1[ICLus][4]*out1[ICLus][4]+out2[ICLus][4]*out2[ICLus][4];
             chi2[2] +=( x1*x1/ex1 );
             ndf[2]++;
         }

         if (out1[ICLus][5] >0 && out2[ICLus][5]>0) {
             x1=out1[ICLus][5]-out2[ICLus][5];
             ex1=out1[ICLus][6]*out1[ICLus][6]+out2[ICLus][6]*out2[ICLus][6];
             chi2[3] += (x1*x1/ex1 );
             ndf[3]++;
         }


        if (out1[ICLus][7] >0 && out2[ICLus][7]>0) {
             x1=out1[ICLus][7]-out2[ICLus][7];
             ex1=out1[ICLus][8]*out1[ICLus][8]+out2[ICLus][8]*out2[ICLus][8];
             chi2[4] += (x1*x1/ex1 );
             ndf[4]++;
         }


/*
       double p1= Math.sqrt(xx1)/(pcount1*eSimilarity);
       p1=1.0/Math.exp(p1); 
       double p2= Math.sqrt(xx2)/(pcount2*eSimilarity);
       p2=1.0/Math.exp(p2);
       double p3= Math.sqrt(xx3)/(pcount3*eSimilarity);
       p3=1.0/Math.exp(p3);
       double p4= Math.sqrt(xx4)/(pcount4*eSimilarity);
       p4=1.0/Math.exp(p4);
*/

/*
        p[1]=1.0-math.Probability.chiSquare(pcount[1], xx[1]);
        p[2]=1.0-math.Probability.chiSquare(pcount[2], xx[2]);
        p[3]=1.0-math.Probability.chiSquare(pcount[3], xx[3]);
        p[4]=1.0-math.Probability.chiSquare(pcount[4], xx[4]);
*/

       for (int i=1; i<chi2.length; i++) p[i]=chi2ndf(chi2[i],ndf[i]);
       System.out.println("SO FAR RESULTS=");
       Decision.print(chi2,ndf);

       for (int i=1; i<chi2.length; i++) pSoFar[i][ICLus]=p[i];


       double cut_off0=5; 
       double cut_off1=10;
       double cut_off2=10;
       // difference in geometry
       if (FAST==true && ICLus>1 && p[0]>cut_off0 && p[1]>cut_off1 &&  p[2]>cut_off2) {
          System.out.println("Images are very different stop Prob="+p[1]);
          break;
       } 

       // difference in color
       cut_off0=7;
       if (FAST==true && ICLus>1 && p[0]>cut_off0) {
          System.out.println("Images are very different stop Prob="+p[1]);
          break;
       }




       } // end if (prob calculations) 


       } // end loop over clusters


       if (iREAD==false) {
           writeDoubleArray(new File("sampling1.txt"), out1);
           System.out.println("out1 =  \n" + printDoubleArray(out1));
           System.out.println("Writing  sampling1.txt");
       }


       if (iREAD==true)  {

       for (int j=1; j<ndf.length; j++)  ndf[j]=0;
       for (int j=1; j<chi2.length; j++) chi2[j]=0;
 
        // find min length
       int imin= out1.length;
       if (out2.length<out1.length) imin=out2.length;  

       for (int i=0; i<imin; i++) {
         if (out1[i][1] >0 && out2[i][1]>0) { 
             x1=out1[i][1]-out2[i][1];
             ex1=out1[i][2]*out1[i][2]+out2[i][2]*out2[i][2];  
             chi2[1] +=( x1*x1/ex1 );
             ndf[1]++;
             }
       }

      for (int i=0; i<imin; i++) {
          if (out1[i][3] >0 && out2[i][3]>0) {
             x1=out1[i][3]-out2[i][3];
             ex1=out1[i][4]*out1[i][4]+out2[i][4]*out2[i][4];
             chi2[2] +=( x1*x1/ex1 );
             ndf[2]++;
             }
       }


      for (int i=0; i<imin; i++) {
          if (out1[i][5] >0 && out2[i][5]>0) {
             x1=out1[i][5]-out2[i][5];
             ex1=out1[i][6]*out1[i][6]+out2[i][6]*out2[i][6];
             chi2[3] +=( x1*x1/ex1 );
             ndf[3]++;
             }
       }


       for (int i=0; i<imin; i++) {
        if (out1[i][7] >0 && out2[i][7]>0) {
             x1=out1[i][7]-out2[i][7];
             ex1=out1[i][8]*out1[i][8]+out2[i][8]*out2[i][8];
             chi2[4] +=( x1*x1/ex1 );
             ndf[4]++;
             }
       }

/*
        p[1]=1.0-math.Probability.chiSquare(pcount[1], xx[1]);
        p[2]=1.0-math.Probability.chiSquare(pcount[2], xx[2]);
        p[3]=1.0-math.Probability.chiSquare(pcount[3], xx[3]);
        p[4]=1.0-math.Probability.chiSquare(pcount[4], xx[4]);
*/

         for (int i=1; i<chi2.length; i++) p[i]=chi2ndf(chi2[i],ndf[i]); 



/*

       xx1= Math.sqrt(xx1) / (count1*eSimilarity);
       p1=1.0/Math.exp(xx1);
       xx2= Math.sqrt(xx2) / (count2*eSimilarity);
       p2=1.0/Math.exp(xx2);
       xx3= Math.sqrt(xx3) / (count3*eSimilarity);
       p3=1.0/Math.exp(xx3);
       xx4= Math.sqrt(xx4) / (count4*eSimilarity);
       p4=1.0/Math.exp(xx4);

*/

      sw.stop();
      // System.out.println("out1(produced) =  \n" + printDoubleArray(out2));
      // System.out.println("out2(read) =  \n" + printDoubleArray(out2));
      new Decision(type,chi2,ndf,pSoFar,ifile,Double.toString(sw.getTime()/1000));
      Decision.print(chi2,ndf);

      // System.out.println("pSoFar =  \n" + printDoubleArray( pSoFar ));
     SPlot sp= new SPlot();
     sp.visible(false);
     sp.draw(legend,x,y);
     sp.write(xmlout);


    } // end task 



/*
       // get results ----------------------------- 
       results=clusterer.getDescriptor();
       int nonempty=KMeansRegionClusterer.getNonEmptyClusters(results);
       System.out.println("No of non-empty="+Integer.toString( nonempty ));
       // get largest
       int ilargest=KMeansRegionClusterer.getLargestClusterIndex(results);
       for(int d=0;d<results.length;d++) {
       KMeansClusterDescriptor kc = results[d];
       System.out.println("\nCluster="+Integer.toString(d));
       System.out.println(kc.toString());
       }

      System.out.println("\nLargest Cluster ID="+Integer.toString( ilargest ) );
     // clusterer.printClusterIndex();
      System.out.println("\nGlobal XY compactness="+Double.toString(clusterer.getQualityXY()));
      System.out.println("\nGlobal Color compactness="+Double.toString(clusterer.getQualityCol()));


       TiledImage tim=clusterer.getOutput();
       Graphics2D g2 = tim.createGraphics();

       Ellipse2D circle=null;
       for(int d=0;d<results.length;d++) {
       // do not plot largest cluster (i.e. background)
       if  ( d == ilargest) continue;
 
       KMeansClusterDescriptor kc = results[d];
       float[] centroid= kc.getCentroid();
       float centroidX= kc.getCentroidX();
       float centroidY= kc.getCentroidY();
//       System.out.println("Cluster x="+Float.toString(centroidX)+
//                          " y="+Float.toString(centroidY));
       circle = new Ellipse2D.Double(centroidX-4, centroidY-4, 8, 8);
      // g2.setColor(new Color( (int)centroid[0], (int)centroid[1], (int)centroid[2]) );
       g2.setColor(Color.red);
       g2.fill(circle);
       }


        JAI.create("filestore",tim,ofile,"TIFF");
*/

    }


  /**
  * get measure of difference  
  */
 public double  chi2ndf(double chi2, int ndf) { 
    double x=-999;
    if (ndf>0) x=chi2/ndf; 
    // double p=StatTools.chi2Df(chi2,ndf);
    // double p=math.Probability.chiSquare(ndf, chi2);

    return x; 
 }


 /**
  * The application entry point, which will need an input image file name and
  * an output image file name.
  */
  public static void main(String[] args)
    {

      if (args.length>1) {
          new testMain(args[0],args[1]); 
        } else {
          System.out.println("testMain <file> <write>or<read>");
        }

    }
  
  }
